@@ -37,9 +37,11 @@ pysqlite_connection_init(PyObject *self, PyObject *args, PyObject *kwargs)
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(database), &_Py_ID(timeout), &_Py_ID(detect_types), &_Py_ID(isolation_level), &_Py_ID(check_same_thread), &_Py_ID(factory), &_Py_ID(cached_statements), &_Py_ID(uri), &_Py_ID(autocommit), },
     };
     #undef NUM_KEYWORDS
@@ -81,7 +83,8 @@ pysqlite_connection_init(PyObject *self, PyObject *args, PyObject *kwargs)
             goto exit;
         }
     }
-    fastargs = _PyArg_UnpackKeywords(_PyTuple_CAST(args)->ob_item, nargs, kwargs, NULL, &_parser, 1, 8, 0, argsbuf);
+    fastargs = _PyArg_UnpackKeywords(_PyTuple_CAST(args)->ob_item, nargs, kwargs, NULL, &_parser,
+            /*minpos*/ 1, /*maxpos*/ 8, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!fastargs) {
         goto exit;
     }
@@ -181,7 +184,7 @@ static PyObject *
 pysqlite_connection_cursor_impl(pysqlite_Connection *self, PyObject *factory);
 
 static PyObject *
-pysqlite_connection_cursor(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_cursor(PyObject *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -190,9 +193,11 @@ pysqlite_connection_cursor(pysqlite_Connection *self, PyObject *const *args, Py_
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(factory), },
     };
     #undef NUM_KEYWORDS
@@ -213,7 +218,8 @@ pysqlite_connection_cursor(pysqlite_Connection *self, PyObject *const *args, Py_
     Py_ssize_t noptargs = nargs + (kwnames ? PyTuple_GET_SIZE(kwnames) : 0) - 0;
     PyObject *factory = NULL;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 0, 1, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 0, /*maxpos*/ 1, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -222,7 +228,7 @@ pysqlite_connection_cursor(pysqlite_Connection *self, PyObject *const *args, Py_
     }
     factory = args[0];
 skip_optional_pos:
-    return_value = pysqlite_connection_cursor_impl(self, factory);
+    return_value = pysqlite_connection_cursor_impl((pysqlite_Connection *)self, factory);
 
 exit:
     return return_value;
@@ -253,7 +259,7 @@ blobopen_impl(pysqlite_Connection *self, const char *table, const char *col,
               sqlite3_int64 row, int readonly, const char *name);
 
 static PyObject *
-blobopen(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+blobopen(PyObject *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -262,9 +268,11 @@ blobopen(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyO
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(readonly), &_Py_ID(name), },
     };
     #undef NUM_KEYWORDS
@@ -289,7 +297,8 @@ blobopen(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyO
     int readonly = 0;
     const char *name = "main";
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 3, 3, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 3, /*maxpos*/ 3, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -348,7 +357,7 @@ blobopen(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyO
         goto exit;
     }
 skip_optional_kwonly:
-    return_value = blobopen_impl(self, table, col, row, readonly, name);
+    return_value = blobopen_impl((pysqlite_Connection *)self, table, col, row, readonly, name);
 
 exit:
     return return_value;
@@ -369,9 +378,9 @@ static PyObject *
 pysqlite_connection_close_impl(pysqlite_Connection *self);
 
 static PyObject *
-pysqlite_connection_close(pysqlite_Connection *self, PyObject *Py_UNUSED(ignored))
+pysqlite_connection_close(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return pysqlite_connection_close_impl(self);
+    return pysqlite_connection_close_impl((pysqlite_Connection *)self);
 }
 
 PyDoc_STRVAR(pysqlite_connection_commit__doc__,
@@ -389,9 +398,9 @@ static PyObject *
 pysqlite_connection_commit_impl(pysqlite_Connection *self);
 
 static PyObject *
-pysqlite_connection_commit(pysqlite_Connection *self, PyObject *Py_UNUSED(ignored))
+pysqlite_connection_commit(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return pysqlite_connection_commit_impl(self);
+    return pysqlite_connection_commit_impl((pysqlite_Connection *)self);
 }
 
 PyDoc_STRVAR(pysqlite_connection_rollback__doc__,
@@ -409,9 +418,9 @@ static PyObject *
 pysqlite_connection_rollback_impl(pysqlite_Connection *self);
 
 static PyObject *
-pysqlite_connection_rollback(pysqlite_Connection *self, PyObject *Py_UNUSED(ignored))
+pysqlite_connection_rollback(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return pysqlite_connection_rollback_impl(self);
+    return pysqlite_connection_rollback_impl((pysqlite_Connection *)self);
 }
 
 PyDoc_STRVAR(pysqlite_connection_create_function__doc__,
@@ -446,7 +455,7 @@ pysqlite_connection_create_function_impl(pysqlite_Connection *self,
 #endif
 
 static PyObject *
-pysqlite_connection_create_function(pysqlite_Connection *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_create_function(PyObject *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -455,9 +464,11 @@ pysqlite_connection_create_function(pysqlite_Connection *self, PyTypeObject *cls
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(name), &_Py_ID(narg), &_Py_ID(func), &_Py_ID(deterministic), },
     };
     #undef NUM_KEYWORDS
@@ -481,7 +492,8 @@ pysqlite_connection_create_function(pysqlite_Connection *self, PyTypeObject *cls
     PyObject *func;
     int deterministic = 0;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 3, 3, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 3, /*maxpos*/ 3, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -521,7 +533,7 @@ pysqlite_connection_create_function(pysqlite_Connection *self, PyTypeObject *cls
         goto exit;
     }
 skip_optional_kwonly:
-    return_value = pysqlite_connection_create_function_impl(self, cls, name, narg, func, deterministic);
+    return_value = pysqlite_connection_create_function_impl((pysqlite_Connection *)self, cls, name, narg, func, deterministic);
 
 exit:
     return return_value;
@@ -553,7 +565,7 @@ create_window_function_impl(pysqlite_Connection *self, PyTypeObject *cls,
                             PyObject *aggregate_class);
 
 static PyObject *
-create_window_function(pysqlite_Connection *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+create_window_function(PyObject *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -574,7 +586,8 @@ create_window_function(pysqlite_Connection *self, PyTypeObject *cls, PyObject *c
     int num_params;
     PyObject *aggregate_class;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 3, 3, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 3, /*maxpos*/ 3, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -596,7 +609,7 @@ create_window_function(pysqlite_Connection *self, PyTypeObject *cls, PyObject *c
         goto exit;
     }
     aggregate_class = args[2];
-    return_value = create_window_function_impl(self, cls, name, num_params, aggregate_class);
+    return_value = create_window_function_impl((pysqlite_Connection *)self, cls, name, num_params, aggregate_class);
 
 exit:
     return return_value;
@@ -637,7 +650,7 @@ pysqlite_connection_create_aggregate_impl(pysqlite_Connection *self,
 #endif
 
 static PyObject *
-pysqlite_connection_create_aggregate(pysqlite_Connection *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_create_aggregate(PyObject *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -646,9 +659,11 @@ pysqlite_connection_create_aggregate(pysqlite_Connection *self, PyTypeObject *cl
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(name), &_Py_ID(n_arg), &_Py_ID(aggregate_class), },
     };
     #undef NUM_KEYWORDS
@@ -670,7 +685,8 @@ pysqlite_connection_create_aggregate(pysqlite_Connection *self, PyTypeObject *cl
     int n_arg;
     PyObject *aggregate_class;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 3, 3, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 3, /*maxpos*/ 3, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -702,7 +718,7 @@ pysqlite_connection_create_aggregate(pysqlite_Connection *self, PyTypeObject *cl
         goto exit;
     }
     aggregate_class = args[2];
-    return_value = pysqlite_connection_create_aggregate_impl(self, cls, name, n_arg, aggregate_class);
+    return_value = pysqlite_connection_create_aggregate_impl((pysqlite_Connection *)self, cls, name, n_arg, aggregate_class);
 
 exit:
     return return_value;
@@ -739,7 +755,7 @@ pysqlite_connection_set_authorizer_impl(pysqlite_Connection *self,
 #endif
 
 static PyObject *
-pysqlite_connection_set_authorizer(pysqlite_Connection *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_set_authorizer(PyObject *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -748,9 +764,11 @@ pysqlite_connection_set_authorizer(pysqlite_Connection *self, PyTypeObject *cls,
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(authorizer_callback), },
     };
     #undef NUM_KEYWORDS
@@ -770,7 +788,8 @@ pysqlite_connection_set_authorizer(pysqlite_Connection *self, PyTypeObject *cls,
     PyObject *argsbuf[1];
     PyObject *callable;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 1, 1, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 1, /*maxpos*/ 1, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -785,7 +804,7 @@ pysqlite_connection_set_authorizer(pysqlite_Connection *self, PyTypeObject *cls,
         }
     }
     callable = args[0];
-    return_value = pysqlite_connection_set_authorizer_impl(self, cls, callable);
+    return_value = pysqlite_connection_set_authorizer_impl((pysqlite_Connection *)self, cls, callable);
 
 exit:
     return return_value;
@@ -832,7 +851,7 @@ pysqlite_connection_set_progress_handler_impl(pysqlite_Connection *self,
 #endif
 
 static PyObject *
-pysqlite_connection_set_progress_handler(pysqlite_Connection *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_set_progress_handler(PyObject *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -841,9 +860,11 @@ pysqlite_connection_set_progress_handler(pysqlite_Connection *self, PyTypeObject
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(progress_handler), _Py_LATIN1_CHR('n'), },
     };
     #undef NUM_KEYWORDS
@@ -864,7 +885,8 @@ pysqlite_connection_set_progress_handler(pysqlite_Connection *self, PyTypeObject
     PyObject *callable;
     int n;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 2, 2, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 2, /*maxpos*/ 2, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -883,7 +905,7 @@ pysqlite_connection_set_progress_handler(pysqlite_Connection *self, PyTypeObject
     if (n == -1 && PyErr_Occurred()) {
         goto exit;
     }
-    return_value = pysqlite_connection_set_progress_handler_impl(self, cls, callable, n);
+    return_value = pysqlite_connection_set_progress_handler_impl((pysqlite_Connection *)self, cls, callable, n);
 
 exit:
     return return_value;
@@ -920,7 +942,7 @@ pysqlite_connection_set_trace_callback_impl(pysqlite_Connection *self,
 #endif
 
 static PyObject *
-pysqlite_connection_set_trace_callback(pysqlite_Connection *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_set_trace_callback(PyObject *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -929,9 +951,11 @@ pysqlite_connection_set_trace_callback(pysqlite_Connection *self, PyTypeObject *
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(trace_callback), },
     };
     #undef NUM_KEYWORDS
@@ -951,7 +975,8 @@ pysqlite_connection_set_trace_callback(pysqlite_Connection *self, PyTypeObject *
     PyObject *argsbuf[1];
     PyObject *callable;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 1, 1, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 1, /*maxpos*/ 1, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -966,7 +991,7 @@ pysqlite_connection_set_trace_callback(pysqlite_Connection *self, PyTypeObject *
         }
     }
     callable = args[0];
-    return_value = pysqlite_connection_set_trace_callback_impl(self, cls, callable);
+    return_value = pysqlite_connection_set_trace_callback_impl((pysqlite_Connection *)self, cls, callable);
 
 exit:
     return return_value;
@@ -988,7 +1013,7 @@ pysqlite_connection_enable_load_extension_impl(pysqlite_Connection *self,
                                                int onoff);
 
 static PyObject *
-pysqlite_connection_enable_load_extension(pysqlite_Connection *self, PyObject *arg)
+pysqlite_connection_enable_load_extension(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     int onoff;
@@ -997,7 +1022,7 @@ pysqlite_connection_enable_load_extension(pysqlite_Connection *self, PyObject *a
     if (onoff < 0) {
         goto exit;
     }
-    return_value = pysqlite_connection_enable_load_extension_impl(self, onoff);
+    return_value = pysqlite_connection_enable_load_extension_impl((pysqlite_Connection *)self, onoff);
 
 exit:
     return return_value;
@@ -1022,7 +1047,7 @@ pysqlite_connection_load_extension_impl(pysqlite_Connection *self,
                                         const char *entrypoint);
 
 static PyObject *
-pysqlite_connection_load_extension(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_load_extension(PyObject *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -1031,9 +1056,11 @@ pysqlite_connection_load_extension(pysqlite_Connection *self, PyObject *const *a
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(entrypoint), },
     };
     #undef NUM_KEYWORDS
@@ -1055,7 +1082,8 @@ pysqlite_connection_load_extension(pysqlite_Connection *self, PyObject *const *a
     const char *extension_name;
     const char *entrypoint = NULL;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 1, 1, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 1, /*maxpos*/ 1, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -1094,7 +1122,7 @@ pysqlite_connection_load_extension(pysqlite_Connection *self, PyObject *const *a
         goto exit;
     }
 skip_optional_kwonly:
-    return_value = pysqlite_connection_load_extension_impl(self, extension_name, entrypoint);
+    return_value = pysqlite_connection_load_extension_impl((pysqlite_Connection *)self, extension_name, entrypoint);
 
 exit:
     return return_value;
@@ -1116,7 +1144,7 @@ pysqlite_connection_execute_impl(pysqlite_Connection *self, PyObject *sql,
                                  PyObject *parameters);
 
 static PyObject *
-pysqlite_connection_execute(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs)
+pysqlite_connection_execute(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     PyObject *sql;
@@ -1135,7 +1163,7 @@ pysqlite_connection_execute(pysqlite_Connection *self, PyObject *const *args, Py
     }
     parameters = args[1];
 skip_optional:
-    return_value = pysqlite_connection_execute_impl(self, sql, parameters);
+    return_value = pysqlite_connection_execute_impl((pysqlite_Connection *)self, sql, parameters);
 
 exit:
     return return_value;
@@ -1155,7 +1183,7 @@ pysqlite_connection_executemany_impl(pysqlite_Connection *self,
                                      PyObject *sql, PyObject *parameters);
 
 static PyObject *
-pysqlite_connection_executemany(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs)
+pysqlite_connection_executemany(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     PyObject *sql;
@@ -1170,7 +1198,7 @@ pysqlite_connection_executemany(pysqlite_Connection *self, PyObject *const *args
     }
     sql = args[0];
     parameters = args[1];
-    return_value = pysqlite_connection_executemany_impl(self, sql, parameters);
+    return_value = pysqlite_connection_executemany_impl((pysqlite_Connection *)self, sql, parameters);
 
 exit:
     return return_value;
@@ -1185,6 +1213,20 @@ PyDoc_STRVAR(pysqlite_connection_executescript__doc__,
 #define PYSQLITE_CONNECTION_EXECUTESCRIPT_METHODDEF    \
     {"executescript", (PyCFunction)pysqlite_connection_executescript, METH_O, pysqlite_connection_executescript__doc__},
 
+static PyObject *
+pysqlite_connection_executescript_impl(pysqlite_Connection *self,
+                                       PyObject *script_obj);
+
+static PyObject *
+pysqlite_connection_executescript(PyObject *self, PyObject *script_obj)
+{
+    PyObject *return_value = NULL;
+
+    return_value = pysqlite_connection_executescript_impl((pysqlite_Connection *)self, script_obj);
+
+    return return_value;
+}
+
 PyDoc_STRVAR(pysqlite_connection_interrupt__doc__,
 "interrupt($self, /)\n"
 "--\n"
@@ -1198,9 +1240,9 @@ static PyObject *
 pysqlite_connection_interrupt_impl(pysqlite_Connection *self);
 
 static PyObject *
-pysqlite_connection_interrupt(pysqlite_Connection *self, PyObject *Py_UNUSED(ignored))
+pysqlite_connection_interrupt(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return pysqlite_connection_interrupt_impl(self);
+    return pysqlite_connection_interrupt_impl((pysqlite_Connection *)self);
 }
 
 PyDoc_STRVAR(pysqlite_connection_iterdump__doc__,
@@ -1220,7 +1262,7 @@ pysqlite_connection_iterdump_impl(pysqlite_Connection *self,
                                   PyObject *filter);
 
 static PyObject *
-pysqlite_connection_iterdump(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_iterdump(PyObject *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -1229,9 +1271,11 @@ pysqlite_connection_iterdump(pysqlite_Connection *self, PyObject *const *args, P
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(filter), },
     };
     #undef NUM_KEYWORDS
@@ -1252,7 +1296,8 @@ pysqlite_connection_iterdump(pysqlite_Connection *self, PyObject *const *args, P
     Py_ssize_t noptargs = nargs + (kwnames ? PyTuple_GET_SIZE(kwnames) : 0) - 0;
     PyObject *filter = Py_None;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 0, 0, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 0, /*maxpos*/ 0, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -1261,7 +1306,7 @@ pysqlite_connection_iterdump(pysqlite_Connection *self, PyObject *const *args, P
     }
     filter = args[0];
 skip_optional_kwonly:
-    return_value = pysqlite_connection_iterdump_impl(self, filter);
+    return_value = pysqlite_connection_iterdump_impl((pysqlite_Connection *)self, filter);
 
 exit:
     return return_value;
@@ -1284,7 +1329,7 @@ pysqlite_connection_backup_impl(pysqlite_Connection *self,
                                 double sleep);
 
 static PyObject *
-pysqlite_connection_backup(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_backup(PyObject *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -1293,9 +1338,11 @@ pysqlite_connection_backup(pysqlite_Connection *self, PyObject *const *args, Py_
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(target), &_Py_ID(pages), &_Py_ID(progress), &_Py_ID(name), &_Py_ID(sleep), },
     };
     #undef NUM_KEYWORDS
@@ -1320,7 +1367,8 @@ pysqlite_connection_backup(pysqlite_Connection *self, PyObject *const *args, Py_
     const char *name = "main";
     double sleep = 0.25;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 1, 1, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 1, /*maxpos*/ 1, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -1376,7 +1424,7 @@ pysqlite_connection_backup(pysqlite_Connection *self, PyObject *const *args, Py_
         }
     }
 skip_optional_kwonly:
-    return_value = pysqlite_connection_backup_impl(self, target, pages, progress, name, sleep);
+    return_value = pysqlite_connection_backup_impl((pysqlite_Connection *)self, target, pages, progress, name, sleep);
 
 exit:
     return return_value;
@@ -1398,7 +1446,7 @@ pysqlite_connection_create_collation_impl(pysqlite_Connection *self,
                                           PyObject *callable);
 
 static PyObject *
-pysqlite_connection_create_collation(pysqlite_Connection *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+pysqlite_connection_create_collation(PyObject *self, PyTypeObject *cls, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -1418,7 +1466,8 @@ pysqlite_connection_create_collation(pysqlite_Connection *self, PyTypeObject *cl
     const char *name;
     PyObject *callable;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 2, 2, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 2, /*maxpos*/ 2, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -1436,7 +1485,7 @@ pysqlite_connection_create_collation(pysqlite_Connection *self, PyTypeObject *cl
         goto exit;
     }
     callable = args[1];
-    return_value = pysqlite_connection_create_collation_impl(self, cls, name, callable);
+    return_value = pysqlite_connection_create_collation_impl((pysqlite_Connection *)self, cls, name, callable);
 
 exit:
     return return_value;
@@ -1465,7 +1514,7 @@ static PyObject *
 serialize_impl(pysqlite_Connection *self, const char *name);
 
 static PyObject *
-serialize(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+serialize(PyObject *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -1474,9 +1523,11 @@ serialize(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, Py
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(name), },
     };
     #undef NUM_KEYWORDS
@@ -1497,7 +1548,8 @@ serialize(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, Py
     Py_ssize_t noptargs = nargs + (kwnames ? PyTuple_GET_SIZE(kwnames) : 0) - 0;
     const char *name = "main";
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 0, 0, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 0, /*maxpos*/ 0, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -1518,7 +1570,7 @@ serialize(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, Py
         goto exit;
     }
 skip_optional_kwonly:
-    return_value = serialize_impl(self, name);
+    return_value = serialize_impl((pysqlite_Connection *)self, name);
 
 exit:
     return return_value;
@@ -1554,7 +1606,7 @@ deserialize_impl(pysqlite_Connection *self, Py_buffer *data,
                  const char *name);
 
 static PyObject *
-deserialize(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+deserialize(PyObject *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -1563,9 +1615,11 @@ deserialize(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs,
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(name), },
     };
     #undef NUM_KEYWORDS
@@ -1587,7 +1641,8 @@ deserialize(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs,
     Py_buffer data = {NULL, NULL};
     const char *name = "main";
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 1, 1, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 1, /*maxpos*/ 1, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -1623,7 +1678,7 @@ deserialize(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs,
         goto exit;
     }
 skip_optional_kwonly:
-    return_value = deserialize_impl(self, &data, name);
+    return_value = deserialize_impl((pysqlite_Connection *)self, &data, name);
 
 exit:
     /* Cleanup for data */
@@ -1651,9 +1706,9 @@ static PyObject *
 pysqlite_connection_enter_impl(pysqlite_Connection *self);
 
 static PyObject *
-pysqlite_connection_enter(pysqlite_Connection *self, PyObject *Py_UNUSED(ignored))
+pysqlite_connection_enter(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return pysqlite_connection_enter_impl(self);
+    return pysqlite_connection_enter_impl((pysqlite_Connection *)self);
 }
 
 PyDoc_STRVAR(pysqlite_connection_exit__doc__,
@@ -1672,7 +1727,7 @@ pysqlite_connection_exit_impl(pysqlite_Connection *self, PyObject *exc_type,
                               PyObject *exc_value, PyObject *exc_tb);
 
 static PyObject *
-pysqlite_connection_exit(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs)
+pysqlite_connection_exit(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     PyObject *exc_type;
@@ -1685,7 +1740,7 @@ pysqlite_connection_exit(pysqlite_Connection *self, PyObject *const *args, Py_ss
     exc_type = args[0];
     exc_value = args[1];
     exc_tb = args[2];
-    return_value = pysqlite_connection_exit_impl(self, exc_type, exc_value, exc_tb);
+    return_value = pysqlite_connection_exit_impl((pysqlite_Connection *)self, exc_type, exc_value, exc_tb);
 
 exit:
     return return_value;
@@ -1714,7 +1769,7 @@ static PyObject *
 setlimit_impl(pysqlite_Connection *self, int category, int limit);
 
 static PyObject *
-setlimit(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs)
+setlimit(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     int category;
@@ -1731,7 +1786,7 @@ setlimit(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs)
     if (limit == -1 && PyErr_Occurred()) {
         goto exit;
     }
-    return_value = setlimit_impl(self, category, limit);
+    return_value = setlimit_impl((pysqlite_Connection *)self, category, limit);
 
 exit:
     return return_value;
@@ -1753,7 +1808,7 @@ static PyObject *
 getlimit_impl(pysqlite_Connection *self, int category);
 
 static PyObject *
-getlimit(pysqlite_Connection *self, PyObject *arg)
+getlimit(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     int category;
@@ -1762,7 +1817,7 @@ getlimit(pysqlite_Connection *self, PyObject *arg)
     if (category == -1 && PyErr_Occurred()) {
         goto exit;
     }
-    return_value = getlimit_impl(self, category);
+    return_value = getlimit_impl((pysqlite_Connection *)self, category);
 
 exit:
     return return_value;
@@ -1784,7 +1839,7 @@ static PyObject *
 setconfig_impl(pysqlite_Connection *self, int op, int enable);
 
 static PyObject *
-setconfig(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs)
+setconfig(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     int op;
@@ -1805,7 +1860,7 @@ setconfig(pysqlite_Connection *self, PyObject *const *args, Py_ssize_t nargs)
         goto exit;
     }
 skip_optional:
-    return_value = setconfig_impl(self, op, enable);
+    return_value = setconfig_impl((pysqlite_Connection *)self, op, enable);
 
 exit:
     return return_value;
@@ -1827,7 +1882,7 @@ static int
 getconfig_impl(pysqlite_Connection *self, int op);
 
 static PyObject *
-getconfig(pysqlite_Connection *self, PyObject *arg)
+getconfig(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     int op;
@@ -1837,7 +1892,7 @@ getconfig(pysqlite_Connection *self, PyObject *arg)
     if (op == -1 && PyErr_Occurred()) {
         goto exit;
     }
-    _return_value = getconfig_impl(self, op);
+    _return_value = getconfig_impl((pysqlite_Connection *)self, op);
     if ((_return_value == -1) && PyErr_Occurred()) {
         goto exit;
     }
@@ -1866,4 +1921,4 @@ exit:
 #ifndef DESERIALIZE_METHODDEF
     #define DESERIALIZE_METHODDEF
 #endif /* !defined(DESERIALIZE_METHODDEF) */
-/*[clinic end generated code: output=fa34f4c5f8837a7c input=a9049054013a1b77]*/
+/*[clinic end generated code: output=7b502667df24ee41 input=a9049054013a1b77]*/
