@@ -4,8 +4,10 @@
 #include "pycore_abstract.h"      // _PyObject_HasLen()
 #include "pycore_call.h"          // _PyObject_CallNoArgs()
 #include "pycore_ceval.h"         // _PyEval_GetBuiltin()
+#include "pycore_genobject.h"     // _PyCoro_GetAwaitableIter()
 #include "pycore_object.h"        // _PyObject_GC_TRACK()
 
+
 typedef struct {
     PyObject_HEAD
     Py_ssize_t it_index;
@@ -31,16 +33,18 @@ PySeqIter_New(PyObject *seq)
 }
 
 static void
-iter_dealloc(seqiterobject *it)
+iter_dealloc(PyObject *op)
 {
+    seqiterobject *it = (seqiterobject*)op;
     _PyObject_GC_UNTRACK(it);
     Py_XDECREF(it->it_seq);
     PyObject_GC_Del(it);
 }
 
 static int
-iter_traverse(seqiterobject *it, visitproc visit, void *arg)
+iter_traverse(PyObject *op, visitproc visit, void *arg)
 {
+    seqiterobject *it = (seqiterobject*)op;
     Py_VISIT(it->it_seq);
     return 0;
 }
@@ -79,8 +83,9 @@ iter_iternext(PyObject *iterator)
 }
 
 static PyObject *
-iter_len(seqiterobject *it, PyObject *Py_UNUSED(ignored))
+iter_len(PyObject *op, PyObject *Py_UNUSED(ignored))
 {
+    seqiterobject *it = (seqiterobject*)op;
     Py_ssize_t seqsize, len;
 
     if (it->it_seq) {
@@ -102,8 +107,9 @@ iter_len(seqiterobject *it, PyObject *Py_UNUSED(ignored))
 PyDoc_STRVAR(length_hint_doc, "Private method returning an estimate of len(list(it)).");
 
 static PyObject *
-iter_reduce(seqiterobject *it, PyObject *Py_UNUSED(ignored))
+iter_reduce(PyObject *op, PyObject *Py_UNUSED(ignored))
 {
+    seqiterobject *it = (seqiterobject*)op;
     PyObject *iter = _PyEval_GetBuiltin(&_Py_ID(iter));
 
     /* _PyEval_GetBuiltin can invoke arbitrary code,
@@ -119,8 +125,9 @@ iter_reduce(seqiterobject *it, PyObject *Py_UNUSED(ignored))
 PyDoc_STRVAR(reduce_doc, "Return state information for pickling.");
 
 static PyObject *
-iter_setstate(seqiterobject *it, PyObject *state)
+iter_setstate(PyObject *op, PyObject *state)
 {
+    seqiterobject *it = (seqiterobject*)op;
     Py_ssize_t index = PyLong_AsSsize_t(state);
     if (index == -1 && PyErr_Occurred())
         return NULL;
@@ -135,9 +142,9 @@ iter_setstate(seqiterobject *it, PyObject *state)
 PyDoc_STRVAR(setstate_doc, "Set state information for unpickling.");
 
 static PyMethodDef seqiter_methods[] = {
-    {"__length_hint__", (PyCFunction)iter_len, METH_NOARGS, length_hint_doc},
-    {"__reduce__", (PyCFunction)iter_reduce, METH_NOARGS, reduce_doc},
-    {"__setstate__", (PyCFunction)iter_setstate, METH_O, setstate_doc},
+    {"__length_hint__", iter_len, METH_NOARGS, length_hint_doc},
+    {"__reduce__", iter_reduce, METH_NOARGS, reduce_doc},
+    {"__setstate__", iter_setstate, METH_O, setstate_doc},
     {NULL,              NULL}           /* sentinel */
 };
 
@@ -147,7 +154,7 @@ PyTypeObject PySeqIter_Type = {
     sizeof(seqiterobject),                      /* tp_basicsize */
     0,                                          /* tp_itemsize */
     /* methods */
-    (destructor)iter_dealloc,                   /* tp_dealloc */
+    iter_dealloc,                               /* tp_dealloc */
     0,                                          /* tp_vectorcall_offset */
     0,                                          /* tp_getattr */
     0,                                          /* tp_setattr */
@@ -164,7 +171,7 @@ PyTypeObject PySeqIter_Type = {
     0,                                          /* tp_as_buffer */
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,    /* tp_flags */
     0,                                          /* tp_doc */
-    (traverseproc)iter_traverse,                /* tp_traverse */
+    iter_traverse,                              /* tp_traverse */
     0,                                          /* tp_clear */
     0,                                          /* tp_richcompare */
     0,                                          /* tp_weaklistoffset */
@@ -195,8 +202,9 @@ PyCallIter_New(PyObject *callable, PyObject *sentinel)
     return (PyObject *)it;
 }
 static void
-calliter_dealloc(calliterobject *it)
+calliter_dealloc(PyObject *op)
 {
+    calliterobject *it = (calliterobject*)op;
     _PyObject_GC_UNTRACK(it);
     Py_XDECREF(it->it_callable);
     Py_XDECREF(it->it_sentinel);
@@ -204,16 +212,18 @@ calliter_dealloc(calliterobject *it)
 }
 
 static int
-calliter_traverse(calliterobject *it, visitproc visit, void *arg)
+calliter_traverse(PyObject *op, visitproc visit, void *arg)
 {
+    calliterobject *it = (calliterobject*)op;
     Py_VISIT(it->it_callable);
     Py_VISIT(it->it_sentinel);
     return 0;
 }
 
 static PyObject *
-calliter_iternext(calliterobject *it)
+calliter_iternext(PyObject *op)
 {
+    calliterobject *it = (calliterobject*)op;
     PyObject *result;
 
     if (it->it_callable == NULL) {
@@ -244,8 +254,9 @@ calliter_iternext(calliterobject *it)
 }
 
 static PyObject *
-calliter_reduce(calliterobject *it, PyObject *Py_UNUSED(ignored))
+calliter_reduce(PyObject *op, PyObject *Py_UNUSED(ignored))
 {
+    calliterobject *it = (calliterobject*)op;
     PyObject *iter = _PyEval_GetBuiltin(&_Py_ID(iter));
 
     /* _PyEval_GetBuiltin can invoke arbitrary code,
@@ -259,7 +270,7 @@ calliter_reduce(calliterobject *it, PyObject *Py_UNUSED(ignored))
 }
 
 static PyMethodDef calliter_methods[] = {
-    {"__reduce__", (PyCFunction)calliter_reduce, METH_NOARGS, reduce_doc},
+    {"__reduce__", calliter_reduce, METH_NOARGS, reduce_doc},
     {NULL,              NULL}           /* sentinel */
 };
 
@@ -269,7 +280,7 @@ PyTypeObject PyCallIter_Type = {
     sizeof(calliterobject),                     /* tp_basicsize */
     0,                                          /* tp_itemsize */
     /* methods */
-    (destructor)calliter_dealloc,               /* tp_dealloc */
+    calliter_dealloc,                           /* tp_dealloc */
     0,                                          /* tp_vectorcall_offset */
     0,                                          /* tp_getattr */
     0,                                          /* tp_setattr */
@@ -286,12 +297,12 @@ PyTypeObject PyCallIter_Type = {
     0,                                          /* tp_as_buffer */
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,    /* tp_flags */
     0,                                          /* tp_doc */
-    (traverseproc)calliter_traverse,            /* tp_traverse */
+    calliter_traverse,                          /* tp_traverse */
     0,                                          /* tp_clear */
     0,                                          /* tp_richcompare */
     0,                                          /* tp_weaklistoffset */
     PyObject_SelfIter,                          /* tp_iter */
-    (iternextfunc)calliter_iternext,            /* tp_iternext */
+    calliter_iternext,                          /* tp_iternext */
     calliter_methods,                           /* tp_methods */
 };
 
@@ -304,9 +315,12 @@ typedef struct {
     PyObject *default_value;
 } anextawaitableobject;
 
+#define anextawaitableobject_CAST(op)   ((anextawaitableobject *)(op))
+
 static void
-anextawaitable_dealloc(anextawaitableobject *obj)
+anextawaitable_dealloc(PyObject *op)
 {
+    anextawaitableobject *obj = anextawaitableobject_CAST(op);
     _PyObject_GC_UNTRACK(obj);
     Py_XDECREF(obj->wrapped);
     Py_XDECREF(obj->default_value);
@@ -314,8 +328,9 @@ anextawaitable_dealloc(anextawaitableobject *obj)
 }
 
 static int
-anextawaitable_traverse(anextawaitableobject *obj, visitproc visit, void *arg)
+anextawaitable_traverse(PyObject *op, visitproc visit, void *arg)
 {
+    anextawaitableobject *obj = anextawaitableobject_CAST(op);
     Py_VISIT(obj->wrapped);
     Py_VISIT(obj->default_value);
     return 0;
@@ -352,7 +367,7 @@ anextawaitable_getiter(anextawaitableobject *obj)
 }
 
 static PyObject *
-anextawaitable_iternext(anextawaitableobject *obj)
+anextawaitable_iternext(PyObject *op)
 {
     /* Consider the following class:
      *
@@ -374,6 +389,7 @@ anextawaitable_iternext(anextawaitableobject *obj)
      * Then `await anext(gen)` can just call
      * gen.__anext__().__next__()
      */
+    anextawaitableobject *obj = anextawaitableobject_CAST(op);
     PyObject *awaitable = anextawaitable_getiter(obj);
     if (awaitable == NULL) {
         return NULL;
@@ -392,12 +408,17 @@ anextawaitable_iternext(anextawaitableobject *obj)
 
 
 static PyObject *
-anextawaitable_proxy(anextawaitableobject *obj, char *meth, PyObject *arg) {
+anextawaitable_proxy(anextawaitableobject *obj, char *meth, PyObject *arg)
+{
     PyObject *awaitable = anextawaitable_getiter(obj);
     if (awaitable == NULL) {
         return NULL;
     }
-    PyObject *ret = PyObject_CallMethod(awaitable, meth, "O", arg);
+    // When specified, 'arg' may be a tuple (if coming from a METH_VARARGS
+    // method) or a single object (if coming from a METH_O method).
+    PyObject *ret = arg == NULL
+        ? PyObject_CallMethod(awaitable, meth, NULL)
+        : PyObject_CallMethod(awaitable, meth, "O", arg);
     Py_DECREF(awaitable);
     if (ret != NULL) {
         return ret;
@@ -416,20 +437,26 @@ anextawaitable_proxy(anextawaitableobject *obj, char *meth, PyObject *arg) {
 
 
 static PyObject *
-anextawaitable_send(anextawaitableobject *obj, PyObject *arg) {
+anextawaitable_send(PyObject *op, PyObject *arg)
+{
+    anextawaitableobject *obj = anextawaitableobject_CAST(op);
     return anextawaitable_proxy(obj, "send", arg);
 }
 
 
 static PyObject *
-anextawaitable_throw(anextawaitableobject *obj, PyObject *arg) {
-    return anextawaitable_proxy(obj, "throw", arg);
+anextawaitable_throw(PyObject *op, PyObject *args)
+{
+    anextawaitableobject *obj = anextawaitableobject_CAST(op);
+    return anextawaitable_proxy(obj, "throw", args);
 }
 
 
 static PyObject *
-anextawaitable_close(anextawaitableobject *obj, PyObject *arg) {
-    return anextawaitable_proxy(obj, "close", arg);
+anextawaitable_close(PyObject *op, PyObject *Py_UNUSED(dummy))
+{
+    anextawaitableobject *obj = anextawaitableobject_CAST(op);
+    return anextawaitable_proxy(obj, "close", NULL);
 }
 
 
@@ -453,9 +480,9 @@ PyDoc_STRVAR(close_doc,
 
 
 static PyMethodDef anextawaitable_methods[] = {
-    {"send",(PyCFunction)anextawaitable_send, METH_O, send_doc},
-    {"throw",(PyCFunction)anextawaitable_throw, METH_VARARGS, throw_doc},
-    {"close",(PyCFunction)anextawaitable_close, METH_VARARGS, close_doc},
+    {"send", anextawaitable_send, METH_O, send_doc},
+    {"throw", anextawaitable_throw, METH_VARARGS, throw_doc},
+    {"close", anextawaitable_close, METH_NOARGS, close_doc},
     {NULL, NULL}        /* Sentinel */
 };
 
@@ -473,7 +500,7 @@ PyTypeObject _PyAnextAwaitable_Type = {
     sizeof(anextawaitableobject),               /* tp_basicsize */
     0,                                          /* tp_itemsize */
     /* methods */
-    (destructor)anextawaitable_dealloc,         /* tp_dealloc */
+    anextawaitable_dealloc,                     /* tp_dealloc */
     0,                                          /* tp_vectorcall_offset */
     0,                                          /* tp_getattr */
     0,                                          /* tp_setattr */
@@ -490,12 +517,12 @@ PyTypeObject _PyAnextAwaitable_Type = {
     0,                                          /* tp_as_buffer */
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,    /* tp_flags */
     0,                                          /* tp_doc */
-    (traverseproc)anextawaitable_traverse,      /* tp_traverse */
+    anextawaitable_traverse,                    /* tp_traverse */
     0,                                          /* tp_clear */
     0,                                          /* tp_richcompare */
     0,                                          /* tp_weaklistoffset */
     PyObject_SelfIter,                          /* tp_iter */
-    (unaryfunc)anextawaitable_iternext,         /* tp_iternext */
+    anextawaitable_iternext,                    /* tp_iternext */
     anextawaitable_methods,                     /* tp_methods */
 };
 
