@@ -1,21 +1,28 @@
-
 /* Module object implementation */
 
 #include "Python.h"
 #include "pycore_call.h"          // _PyObject_CallNoArgs()
+#include "pycore_dict.h"          // _PyDict_EnablePerThreadRefcounting()
 #include "pycore_fileutils.h"     // _Py_wgetcwd
+#include "pycore_import.h"        // _PyImport_GetNextModuleIndex()
 #include "pycore_interp.h"        // PyInterpreterState.importlib
+#include "pycore_long.h"          // _PyLong_GetOne()
 #include "pycore_modsupport.h"    // _PyModule_CreateInitialized()
 #include "pycore_moduleobject.h"  // _PyModule_GetDef()
 #include "pycore_object.h"        // _PyType_AllocNoTrack
 #include "pycore_pyerrors.h"      // _PyErr_FormatFromCause()
 #include "pycore_pystate.h"       // _PyInterpreterState_GET()
 #include "pycore_sysmodule.h"     // _PySys_GetOptionalAttrString()
+#include "pycore_unicodeobject.h" // _PyUnicode_EqualToASCIIString()
 #include "pycore_weakref.h"       // FT_CLEAR_WEAKREFS()
 
 #include "osdefs.h"               // MAXPATHLEN
 
 
+#define _PyModule_CAST(op) \
+    (assert(PyModule_Check(op)), _Py_CAST(PyModuleObject*, (op)))
+
+
 static PyMemberDef module_members[] = {
     {"__dict__", _Py_T_OBJECT, offsetof(PyModuleObject, md_dict), Py_READONLY},
     {0}
@@ -102,9 +109,7 @@ new_module_notrack(PyTypeObject *mt)
 static void
 track_module(PyModuleObject *m)
 {
-    _PyObject_SetDeferredRefcount(m->md_dict);
-    PyObject_GC_Track(m->md_dict);
-
+    _PyDict_EnablePerThreadRefcounting(m->md_dict);
     _PyObject_SetDeferredRefcount((PyObject *)m);
     PyObject_GC_Track(m);
 }
@@ -226,7 +231,9 @@ _PyModule_CreateInitialized(PyModuleDef* module, int module_api_version)
         return NULL;
     }
     name = _PyImport_ResolveNameWithPackageContext(name);
-    if ((m = (PyModuleObject*)PyModule_New(name)) == NULL)
+
+    m = (PyModuleObject*)PyModule_New(name);
+    if (m == NULL)
         return NULL;
 
     if (module->m_size > 0) {
@@ -601,32 +608,51 @@ PyModule_GetName(PyObject *m)
 }
 
 PyObject*
-PyModule_GetFilenameObject(PyObject *mod)
+_PyModule_GetFilenameObject(PyObject *mod)
 {
+    // We return None to indicate "not found" or "bogus".
     if (!PyModule_Check(mod)) {
         PyErr_BadArgument();
         return NULL;
     }
     PyObject *dict = ((PyModuleObject *)mod)->md_dict;  // borrowed reference
     if (dict == NULL) {
-        goto error;
+        // The module has been tampered with.
+        Py_RETURN_NONE;
     }
     PyObject *fileobj;
-    if (PyDict_GetItemRef(dict, &_Py_ID(__file__), &fileobj) <= 0) {
-        // error or not found
-        goto error;
+    int res = PyDict_GetItemRef(dict, &_Py_ID(__file__), &fileobj);
+    if (res < 0) {
+        return NULL;
+    }
+    if (res == 0) {
+        // __file__ isn't set.  There are several reasons why this might
+        // be so, most of them valid reasons.  If it's the __main__
+        // module then we're running the REPL or with -c.  Otherwise
+        // it's a namespace package or other module with a loader that
+        // isn't disk-based.  It could also be that a user created
+        // a module manually but without manually setting __file__.
+        Py_RETURN_NONE;
     }
     if (!PyUnicode_Check(fileobj)) {
         Py_DECREF(fileobj);
-        goto error;
+        Py_RETURN_NONE;
     }
     return fileobj;
+}
 
-error:
-    if (!PyErr_Occurred()) {
-        PyErr_SetString(PyExc_SystemError, "module filename missing");
+PyObject*
+PyModule_GetFilenameObject(PyObject *mod)
+{
+    PyObject *fileobj = _PyModule_GetFilenameObject(mod);
+    if (fileobj == NULL) {
+        return NULL;
     }
-    return NULL;
+    if (fileobj == Py_None) {
+        PyErr_SetString(PyExc_SystemError, "module filename missing");
+        return NULL;
+    }
+    return fileobj;
 }
 
 const char *
@@ -642,6 +668,37 @@ PyModule_GetFilename(PyObject *m)
     return utf8;
 }
 
+Py_ssize_t
+_PyModule_GetFilenameUTF8(PyObject *mod, char *buffer, Py_ssize_t maxlen)
+{
+    // We "return" an empty string for an invalid module
+    // and for a missing, empty, or invalid filename.
+    assert(maxlen >= 0);
+    Py_ssize_t size = -1;
+    PyObject *filenameobj = _PyModule_GetFilenameObject(mod);
+    if (filenameobj == NULL) {
+        return -1;
+    }
+    if (filenameobj == Py_None) {
+        // It is missing or invalid.
+        buffer[0] = '\0';
+        size = 0;
+    }
+    else {
+        const char *filename = PyUnicode_AsUTF8AndSize(filenameobj, &size);
+        assert(size >= 0);
+        if (size > maxlen) {
+            size = -1;
+            PyErr_SetString(PyExc_ValueError, "__file__ too long");
+        }
+        else {
+            (void)strcpy(buffer, filename);
+        }
+    }
+    Py_DECREF(filenameobj);
+    return size;
+}
+
 PyModuleDef*
 PyModule_GetDef(PyObject* m)
 {
@@ -699,7 +756,8 @@ _PyModule_ClearDict(PyObject *d)
                         PyErr_Clear();
                 }
                 if (PyDict_SetItem(d, key, Py_None) != 0) {
-                    PyErr_FormatUnraisable("Exception ignored on clearing module dict");
+                    PyErr_FormatUnraisable("Exception ignored while "
+                                           "clearing module dict");
                 }
             }
         }
@@ -720,7 +778,8 @@ _PyModule_ClearDict(PyObject *d)
                         PyErr_Clear();
                 }
                 if (PyDict_SetItem(d, key, Py_None) != 0) {
-                    PyErr_FormatUnraisable("Exception ignored on clearing module dict");
+                    PyErr_FormatUnraisable("Exception ignored while "
+                                           "clearing module dict");
                 }
             }
         }
@@ -759,21 +818,25 @@ module___init___impl(PyModuleObject *self, PyObject *name, PyObject *doc)
 }
 
 static void
-module_dealloc(PyModuleObject *m)
+module_dealloc(PyObject *self)
 {
-    int verbose = _Py_GetConfig()->verbose;
+    PyModuleObject *m = _PyModule_CAST(self);
 
     PyObject_GC_UnTrack(m);
+
+    int verbose = _Py_GetConfig()->verbose;
     if (verbose && m->md_name) {
         PySys_FormatStderr("# destroy %U\n", m->md_name);
     }
-    FT_CLEAR_WEAKREFS((PyObject *) m, m->md_weaklist);
+    FT_CLEAR_WEAKREFS(self, m->md_weaklist);
+
     /* bpo-39824: Don't call m_free() if m_size > 0 and md_state=NULL */
     if (m->md_def && m->md_def->m_free
         && (m->md_def->m_size <= 0 || m->md_state != NULL))
     {
         m->md_def->m_free(m);
     }
+
     Py_XDECREF(m->md_dict);
     Py_XDECREF(m->md_name);
     if (m->md_state != NULL)
@@ -782,8 +845,9 @@ module_dealloc(PyModuleObject *m)
 }
 
 static PyObject *
-module_repr(PyModuleObject *m)
+module_repr(PyObject *self)
 {
+    PyModuleObject *m = _PyModule_CAST(self);
     PyInterpreterState *interp = _PyInterpreterState_GET();
     return _PyImport_ImportlibModuleRepr(interp, (PyObject *)m);
 }
@@ -1073,14 +1137,17 @@ done:
 
 
 PyObject*
-_Py_module_getattro(PyModuleObject *m, PyObject *name)
+_Py_module_getattro(PyObject *self, PyObject *name)
 {
+    PyModuleObject *m = _PyModule_CAST(self);
     return _Py_module_getattro_impl(m, name, 0);
 }
 
 static int
-module_traverse(PyModuleObject *m, visitproc visit, void *arg)
+module_traverse(PyObject *self, visitproc visit, void *arg)
 {
+    PyModuleObject *m = _PyModule_CAST(self);
+
     /* bpo-39824: Don't call m_traverse() if m_size > 0 and md_state=NULL */
     if (m->md_def && m->md_def->m_traverse
         && (m->md_def->m_size <= 0 || m->md_state != NULL))
@@ -1089,13 +1156,16 @@ module_traverse(PyModuleObject *m, visitproc visit, void *arg)
         if (res)
             return res;
     }
+
     Py_VISIT(m->md_dict);
     return 0;
 }
 
 static int
-module_clear(PyModuleObject *m)
+module_clear(PyObject *self)
 {
+    PyModuleObject *m = _PyModule_CAST(self);
+
     /* bpo-39824: Don't call m_clear() if m_size > 0 and md_state=NULL */
     if (m->md_def && m->md_def->m_clear
         && (m->md_def->m_size <= 0 || m->md_state != NULL))
@@ -1145,7 +1215,7 @@ static PyMethodDef module_methods[] = {
 };
 
 static PyObject *
-module_get_annotations(PyModuleObject *m, void *Py_UNUSED(ignored))
+module_get_dict(PyModuleObject *m)
 {
     PyObject *dict = PyObject_GetAttr((PyObject *)m, &_Py_ID(__dict__));
     if (dict == NULL) {
@@ -1156,11 +1226,124 @@ module_get_annotations(PyModuleObject *m, void *Py_UNUSED(ignored))
         Py_DECREF(dict);
         return NULL;
     }
+    return dict;
+}
+
+static PyObject *
+module_get_annotate(PyObject *self, void *Py_UNUSED(ignored))
+{
+    PyModuleObject *m = _PyModule_CAST(self);
+
+    PyObject *dict = module_get_dict(m);
+    if (dict == NULL) {
+        return NULL;
+    }
+
+    PyObject *annotate;
+    if (PyDict_GetItemRef(dict, &_Py_ID(__annotate__), &annotate) == 0) {
+        annotate = Py_None;
+        if (PyDict_SetItem(dict, &_Py_ID(__annotate__), annotate) == -1) {
+            Py_CLEAR(annotate);
+        }
+    }
+    Py_DECREF(dict);
+    return annotate;
+}
+
+static int
+module_set_annotate(PyObject *self, PyObject *value, void *Py_UNUSED(ignored))
+{
+    PyModuleObject *m = _PyModule_CAST(self);
+    if (value == NULL) {
+        PyErr_SetString(PyExc_TypeError, "cannot delete __annotate__ attribute");
+        return -1;
+    }
+
+    PyObject *dict = module_get_dict(m);
+    if (dict == NULL) {
+        return -1;
+    }
+
+    if (!Py_IsNone(value) && !PyCallable_Check(value)) {
+        PyErr_SetString(PyExc_TypeError, "__annotate__ must be callable or None");
+        Py_DECREF(dict);
+        return -1;
+    }
+
+    if (PyDict_SetItem(dict, &_Py_ID(__annotate__), value) == -1) {
+        Py_DECREF(dict);
+        return -1;
+    }
+    if (!Py_IsNone(value)) {
+        if (PyDict_Pop(dict, &_Py_ID(__annotations__), NULL) == -1) {
+            Py_DECREF(dict);
+            return -1;
+        }
+    }
+    Py_DECREF(dict);
+    return 0;
+}
+
+static PyObject *
+module_get_annotations(PyObject *self, void *Py_UNUSED(ignored))
+{
+    PyModuleObject *m = _PyModule_CAST(self);
+
+    PyObject *dict = module_get_dict(m);
+    if (dict == NULL) {
+        return NULL;
+    }
 
     PyObject *annotations;
     if (PyDict_GetItemRef(dict, &_Py_ID(__annotations__), &annotations) == 0) {
-        annotations = PyDict_New();
-        if (annotations) {
+        PyObject *spec;
+        if (PyDict_GetItemRef(m->md_dict, &_Py_ID(__spec__), &spec) < 0) {
+            Py_DECREF(dict);
+            return NULL;
+        }
+        bool is_initializing = false;
+        if (spec != NULL) {
+            int rc = _PyModuleSpec_IsInitializing(spec);
+            if (rc < 0) {
+                Py_DECREF(spec);
+                Py_DECREF(dict);
+                return NULL;
+            }
+            Py_DECREF(spec);
+            if (rc) {
+                is_initializing = true;
+            }
+        }
+
+        PyObject *annotate;
+        int annotate_result = PyDict_GetItemRef(dict, &_Py_ID(__annotate__), &annotate);
+        if (annotate_result < 0) {
+            Py_DECREF(dict);
+            return NULL;
+        }
+        if (annotate_result == 1 && PyCallable_Check(annotate)) {
+            PyObject *one = _PyLong_GetOne();
+            annotations = _PyObject_CallOneArg(annotate, one);
+            if (annotations == NULL) {
+                Py_DECREF(annotate);
+                Py_DECREF(dict);
+                return NULL;
+            }
+            if (!PyDict_Check(annotations)) {
+                PyErr_Format(PyExc_TypeError, "__annotate__ returned non-dict of type '%.100s'",
+                             Py_TYPE(annotations)->tp_name);
+                Py_DECREF(annotate);
+                Py_DECREF(annotations);
+                Py_DECREF(dict);
+                return NULL;
+            }
+        }
+        else {
+            annotations = PyDict_New();
+        }
+        Py_XDECREF(annotate);
+        // Do not cache annotations if the module is still initializing
+        if (annotations && !is_initializing) {
             int result = PyDict_SetItem(
                     dict, &_Py_ID(__annotations__), annotations);
             if (result) {
@@ -1173,18 +1356,16 @@ module_get_annotations(PyModuleObject *m, void *Py_UNUSED(ignored))
 }
 
 static int
-module_set_annotations(PyModuleObject *m, PyObject *value, void *Py_UNUSED(ignored))
+module_set_annotations(PyObject *self, PyObject *value, void *Py_UNUSED(ignored))
 {
-    int ret = -1;
-    PyObject *dict = PyObject_GetAttr((PyObject *)m, &_Py_ID(__dict__));
+    PyModuleObject *m = _PyModule_CAST(self);
+
+    PyObject *dict = module_get_dict(m);
     if (dict == NULL) {
         return -1;
     }
-    if (!PyDict_Check(dict)) {
-        PyErr_Format(PyExc_TypeError, "<module>.__dict__ is not a dictionary");
-        goto exit;
-    }
 
+    int ret = -1;
     if (value != NULL) {
         /* set */
         ret = PyDict_SetItem(dict, &_Py_ID(__annotations__), value);
@@ -1200,15 +1381,18 @@ module_set_annotations(PyModuleObject *m, PyObject *value, void *Py_UNUSED(ignor
             ret = 0;
         }
     }
+    if (ret == 0 && PyDict_Pop(dict, &_Py_ID(__annotate__), NULL) < 0) {
+        ret = -1;
+    }
 
-exit:
     Py_DECREF(dict);
     return ret;
 }
 
 
 static PyGetSetDef module_getsets[] = {
-    {"__annotations__", (getter)module_get_annotations, (setter)module_set_annotations},
+    {"__annotations__", module_get_annotations, module_set_annotations},
+    {"__annotate__", module_get_annotate, module_set_annotate},
     {NULL}
 };
 
@@ -1217,26 +1401,26 @@ PyTypeObject PyModule_Type = {
     "module",                                   /* tp_name */
     sizeof(PyModuleObject),                     /* tp_basicsize */
     0,                                          /* tp_itemsize */
-    (destructor)module_dealloc,                 /* tp_dealloc */
+    module_dealloc,                             /* tp_dealloc */
     0,                                          /* tp_vectorcall_offset */
     0,                                          /* tp_getattr */
     0,                                          /* tp_setattr */
     0,                                          /* tp_as_async */
-    (reprfunc)module_repr,                      /* tp_repr */
+    module_repr,                                /* tp_repr */
     0,                                          /* tp_as_number */
     0,                                          /* tp_as_sequence */
     0,                                          /* tp_as_mapping */
     0,                                          /* tp_hash */
     0,                                          /* tp_call */
     0,                                          /* tp_str */
-    (getattrofunc)_Py_module_getattro,          /* tp_getattro */
+    _Py_module_getattro,                        /* tp_getattro */
     PyObject_GenericSetAttr,                    /* tp_setattro */
     0,                                          /* tp_as_buffer */
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC |
         Py_TPFLAGS_BASETYPE,                    /* tp_flags */
     module___init____doc__,                     /* tp_doc */
-    (traverseproc)module_traverse,              /* tp_traverse */
-    (inquiry)module_clear,                      /* tp_clear */
+    module_traverse,                            /* tp_traverse */
+    module_clear,                               /* tp_clear */
     0,                                          /* tp_richcompare */
     offsetof(PyModuleObject, md_weaklist),      /* tp_weaklistoffset */
     0,                                          /* tp_iter */
